@@ -69,9 +69,9 @@ validators.FileRequired = FileRequired
 __all__  = ['Form', 'ValidationError',
             'fields', 'validators', 'widgets', 'html5']
 
-__all__ += fields.__all__
+__all__ += fields.core.__all__
 __all__ += validators.__all__
-__all__ += widgets.__all__
+__all__ += widgets.core.__all__
 __all__ += recaptcha.__all__
 
 if _is_sqlalchemy:
@@ -174,7 +174,7 @@ class Form(BaseForm):
         return csrf_token
 
     def validate_csrf(self, field):
-        if not self.csrf_enabled or request.is_xhr:
+        if not self.csrf_enabled:
             return
 
         csrf_token = session.pop(self.csrf_session_key, None)
