@@ -3,9 +3,25 @@ from __future__ import with_statement
 import re
 
 from flask import Flask, Response, render_template, jsonify
+from flaskext.uploads import UploadSet, IMAGES, TEXT, configure_uploads
 from flaskext.testing import TestCase as _TestCase
 from flaskext.wtf import Form, TextField, FileField, HiddenField, \
-    SubmitField, Required, FieldList
+        SubmitField, Required, FieldList, file_required, file_allowed, html5
+
+class DummyField(object):
+    def __init__(self, data, name='f', label='', id='', type='TextField'):
+        self.data = data
+        self.name = name
+        self.label = label
+        self.id = id
+        self.type = type
+
+    _value       = lambda x: x.data
+    __unicode__  = lambda x: x.data
+    __call__     = lambda x, **k: x.data
+    __iter__     = lambda x: iter(x.data)
+    iter_choices = lambda x: iter(x.data)
+
 
 class TestCase(_TestCase):
     
@@ -52,7 +68,7 @@ class TestCase(_TestCase):
             assert form.csrf_enabled
             assert not form.validate()
             assert not form.validate()
-            return Response("OK")
+            return "OK"
 
         @app.route("/hidden/")
         def hidden():
@@ -75,31 +91,113 @@ class TestCase(_TestCase):
         
         return app
 
+class HTML5Tests(TestCase):
+
+    field = DummyField("name", id="name", name="name")
+
+    def test_url_input(self):
+
+        self.assertEqual(html5.URLInput()(self.field),
+            '<input id="name" name="name" type="url" value="name">')
+
+    def test_search_input(self):
+
+        self.assertEqual(html5.SearchInput()(self.field),
+            '<input id="name" name="name" type="search" value="name">')
+
+    def test_date_input(self):
+
+        self.assertEqual(html5.DateInput()(self.field),
+            '<input id="name" name="name" type="date" value="name">')
+
+    def test_email_input(self):
+
+        self.assertEqual(html5.EmailInput()(self.field),
+            '<input id="name" name="name" type="email" value="name">')
+
+    def test_number_input(self):
+
+        self.assertEqual(html5.NumberInput()(self.field, min=0, max=10),
+            '<input id="name" max="10" min="0" name="name" type="number" value="name">')
+
+    def test_range_input(self):
+
+        self.assertEqual(html5.RangeInput()(self.field, min=0, max=10),
+            '<input id="name" max="10" min="0" name="name" type="range" value="name">')
+
+# FILE UPLOAD TESTS #
+
+images = UploadSet("images", IMAGES)
+text = UploadSet("text", TEXT)
+
+class FileUploadForm(Form):
+
+    upload = FileField("Upload file")
+
+class MultipleFileUploadForm(Form):
+
+    uploads = FieldList(FileField("upload"), min_entries=3)
+
+
+class ImageUploadForm(Form):
+
+    upload = FileField("Upload file", 
+                       validators=[file_required(),
+                                   file_allowed(images)])
+
+class TextUploadForm(Form):
+
+    upload = FileField("Upload file", 
+                       validators=[file_required(),
+                                   file_allowed(text)])
+
+
+
 class TestFileUpload(TestCase):
 
+  
     def create_app(self):
 
-        class FileUploadForm(Form):
-
-            upload = FileField("Upload file")
-
-        class MultipleFileUploadForm(Form):
-
-            uploads = FieldList(FileField("upload"), min_entries=3)
-
-
         app = super(TestFileUpload, self).create_app()
         app.config['CSRF_ENABLED'] = False
+        app.config['UPLOADED_FILES_DEST'] = 'uploads'
+        app.config['UPLOADS_DEFAULT_DEST'] = 'uploads'
+        configure_uploads(app, [images, text])
+
+        @app.route("/upload-image/", methods=("POST",))
+        def upload_image():
+            form = ImageUploadForm()
+            if form.validate_on_submit():
+                return "OK"
+            return "invalid"
+
+        @app.route("/upload-text/", methods=("POST",))
+        def upload_text():
+            form = TextUploadForm()
+            if form.validate_on_submit():
+                return "OK"
+            return "invalid"
+
 
         @app.route("/upload-multiple/", methods=("POST",))
         def upload_multiple():
             form = MultipleFileUploadForm()
             if form.validate_on_submit():
+                assert len(form.uploads.entries) == 3
                 for upload in form.uploads.entries:
                     assert upload.file is not None
 
-            return Response("OK")
+            return "OK"
 
+        @app.route("/upload-multiple-field/", methods=("POST",))
+        def upload_multiple_field():
+            form = MultipleFileFieldUploadForm()
+            if form.validate_on_submit():
+                assert len(form.uploads.files) == 3
+                for upload in form.uploads.files:
+                    assert "flask.png" in upload.filename
+                
+            return "OK"
 
         @app.route("/upload/", methods=("POST",))
         def upload():
@@ -118,7 +216,6 @@ class TestFileUpload(TestCase):
         
         return app
 
-
     def test_multiple_files(self):
 
         fps = [self.app.open_resource("flask.png") for i in xrange(3)]
@@ -129,10 +226,26 @@ class TestFileUpload(TestCase):
     def test_valid_file(self):
         
         with self.app.open_resource("flask.png") as fp:
-            response = self.client.post("/upload/", 
+            response = self.client.post("/upload-image/", 
                 data={'upload' : fp})
 
-        assert "flask.png</h3>" in response.data
+        assert "OK" in response.data
+
+    def test_missing_file(self):
+        
+        response = self.client.post("/upload-image/", 
+                data={'upload' : "test"})
+
+        assert "invalid" in response.data
+
+    def test_invalid_file(self):
+        
+        with self.app.open_resource("flask.png") as fp:
+            response = self.client.post("/upload-text/", 
+                data={'upload' : fp})
+
+        assert "invalid" in response.data
+
 
     def test_invalid_file(self):
         
